package java.time.zone;

import java.io.StreamCorruptedException;
import java.util.Arrays;
import java.util.TreeMap;
import java.util.NavigableMap;
import java.io.ByteArrayInputStream;
import java.util.HashSet;
import java.util.Set;
import java.io.DataInputStream;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.File;
import java.util.concurrent.ConcurrentHashMap;
import java.util.Map;
import java.util.List;

final class TzdbZoneRulesProvider extends ZoneRulesProvider
{
    private List<String> regionIds;
    private String versionId;
    private final Map<String, Object> regionToRules;
    
    public TzdbZoneRulesProvider() {
        this.regionToRules = new ConcurrentHashMap();
        try (final DataInputStream dataInputStream = new DataInputStream(new BufferedInputStream(new FileInputStream(new File(System.getProperty("java.home") + File.separator + "lib", "tzdb.dat"))))) {
            this.load(dataInputStream);
        }
        catch (Exception ex) {
            throw new ZoneRulesException("Unable to load TZDB time-zone rules", ex);
        }
    }
    
    @Override
    protected Set<String> provideZoneIds() {
        return new HashSet(this.regionIds);
    }
    
    @Override
    protected ZoneRules provideRules(String s, boolean b) {
        Object o = this.regionToRules.get(s);
        if (o == null) {
            throw new ZoneRulesException("Unknown time-zone ID: " + s);
        }
        try {
            if ((o instanceof byte[])) {
                o = Ser.read(new DataInputStream(new ByteArrayInputStream((byte[])(byte[])o)));
                this.regionToRules.put(s, o);
            }
            return (ZoneRules)o;
        }
        catch (Exception ex) {
            throw new ZoneRulesException("Invalid binary time-zone data: TZDB:" + s + ", version: " + this.versionId, ex);
        }
    }
    
    @Override
    protected NavigableMap<String, ZoneRules> provideVersions(String s) {
        TreeMap<TzdbZoneRulesProvider, ZoneRules> treeMap = new TreeMap();
        ZoneRules rules = ZoneRulesProvider.getRules(s, false);
        if (rules != null) {
            treeMap.put(this.versionId, rules);
            return treeMap;
        }
        return treeMap;
    }
    
    private void load(DataInputStream dataInputStream) throws Exception {
        if (dataInputStream.readByte() != 1) {
            throw new StreamCorruptedException("File format not recognised");
        }
        if (!"TZDB".equals(dataInputStream.readUTF())) {
            throw new StreamCorruptedException("File format not recognised");
        }
        int short1 = dataInputStream.readShort();
        for (int i = 0; i < short1; ++i) {
            this.versionId = dataInputStream.readUTF();
        }
        int short2 = dataInputStream.readShort();
        String[] a = new String[short2];
        for (int j = 0; j < short2; ++j) {
            a[j] = dataInputStream.readUTF();
        }
        this.regionIds = Arrays.asList(a);
        int short3 = dataInputStream.readShort();
        Object[] array = new Object[short3];
        byte[] b;
        for (int k = 0; k < short3; array[k] = b, ++k) {
            b = new byte[dataInputStream.readShort()];
            dataInputStream.readFully(b);
        }
        for (int l = 0; l < short1; ++l) {
            int short4 = dataInputStream.readShort();
            this.regionToRules.clear();
            for (int n = 0; n < short4; ++n) {
                this.regionToRules.put(a[dataInputStream.readShort()], array[dataInputStream.readShort() & 0xFFFF]);
            }
        }
        return;
    }
    
    @Override
    public String toString() {
        return "TZDB[" + this.versionId + "]";
    }
}
